;;; cus-load.el --- automatically extracted custom dependencies
;;
;;; Code:

(put 'mime-display 'custom-loads '("mm-decode"))
(put 'holidays 'custom-loads '("calendar"))
(put 'SQL 'custom-loads '("sql"))
(put 'refbib 'custom-loads '("refbib"))
(put 'elp 'custom-loads '("elp"))
(put 'eshell-ext 'custom-loads '("esh-ext"))
(put 'ffap 'custom-loads '("ffap"))
(put 'shell 'custom-loads '("dirtrack" "shell" "terminal"))
(put 'locate 'custom-loads '("locate"))
(put 'highlight-changes 'custom-loads '("hilit-chg"))
(put 'nnmail 'custom-loads '("nnmail"))
(put 'chistory 'custom-loads '("chistory"))
(put 'shell-directories 'custom-loads '("shell"))
(put 'idlwave-documentation 'custom-loads '("idlwave"))
(put 'footnote 'custom-loads '("footnote"))
(put 'pcomplete 'custom-loads '("pcmpl-cvs" "pcmpl-gnu" "pcmpl-linux" "pcmpl-rpm" "pcomplete" "em-cmpl"))
(put 'calendar-tex 'custom-loads '("cal-tex"))
(put 'fancy-splash-screen 'custom-loads '("startup"))
(put 'hi-lock-interactive-text-highlighting 'custom-loads '("hi-lock"))
(put 'mail-hist 'custom-loads '("mail-hist"))
(put 'gnus-article-emphasis 'custom-loads '("gnus-art"))
(put 'dunnet 'custom-loads '("dunnet"))
(put 'fortran 'custom-loads '("fortran"))
(put 'eshell-script 'custom-loads '("em-script"))
(put 'feedmail-header 'custom-loads '("feedmail"))
(put 'reftex-table-of-contents-browser 'custom-loads '("reftex-vars"))
(put 'mspools 'custom-loads '("mspools"))
(put 'gnus-article-headers 'custom-loads '("gnus-art" "gnus-sum"))
(put 'f90 'custom-loads '("f90"))
(put 'allout 'custom-loads '("allout"))
(put 'ediff-hook 'custom-loads '("ediff-init" "ediff-mult"))
(put 'woman-faces 'custom-loads '("woman"))
(put 'ps-print-vertical 'custom-loads '("ps-print"))
(put 'supercite-hooks 'custom-loads '("supercite"))
(put 'vhdl-menu 'custom-loads '("vhdl-mode"))
(put 'chinese-calendar 'custom-loads '("cal-china"))
(put 'gnus-newsrc 'custom-loads '("gnus-start"))
(put 'expand 'custom-loads '("expand"))
(put 'bookmark 'custom-loads '("bookmark"))
(put 'icon 'custom-loads '("icon"))
(put 'nnmail-procmail 'custom-loads '("nnmail"))
(put 'desktop 'custom-loads '("desktop"))
(put 'eshell-cmpl 'custom-loads '("em-cmpl"))
(put 'cperl-help-system 'custom-loads '("cperl-mode"))
(put 'ps-print-miscellany 'custom-loads '("ps-print"))
(put 'comint-completion 'custom-loads '("comint"))
(put 'gnus-score-kill 'custom-loads '("gnus-kill" "gnus"))
(put 'ldap 'custom-loads '("ldap"))
(put 'remote-compile 'custom-loads '("rcompile"))
(put 'gnus-visual 'custom-loads '("earcon" "gnus-art" "gnus-audio" "gnus" "smiley-ems"))
(put 'dabbrev 'custom-loads '("dabbrev"))
(put 'completion 'custom-loads '("completion" "iswitchb"))
(put 'cpp 'custom-loads '("cpp"))
(put 'ps-print-background 'custom-loads '("ps-print"))
(put 'auto-show 'custom-loads '("auto-show"))
(put 'gnus-score-expire 'custom-loads '("gnus-kill" "gnus-score"))
(put 'supercite-frames 'custom-loads '("supercite"))
(put 'dirtrack 'custom-loads '("dirtrack"))
(put 'ediff-window 'custom-loads '("ediff-help" "ediff-wind"))
(put 'bruce 'custom-loads '("bruce"))
(put 'windows 'custom-loads '("scroll-all" "window" "follow" "windmove"))
(put 'gnus-exit 'custom-loads '("gnus-group" "gnus"))
(put 'speedbar 'custom-loads '("speedbar"))
(put 'etags 'custom-loads '("etags"))
(put 'f90-indent 'custom-loads '("f90"))
(put 'octave-inferior 'custom-loads '("octave-inf" "octave-mod"))
(put 'ebnf-non-terminal 'custom-loads '("ebnf2ps"))
(put 'ebnf-terminal 'custom-loads '("ebnf2ps"))
(put 'gnus-summary-maneuvering 'custom-loads '("gnus-sum"))
(put 'gnus-start 'custom-loads '("gnus-group" "gnus-int" "gnus-start" "gnus-util" "gnus"))
(put 'dcl 'custom-loads '("dcl-mode"))
(put 'cursor 'custom-loads '("frame"))
(put 'ogonek 'custom-loads '("ogonek"))
(put 'gnus-extract-view 'custom-loads '("gnus-sum" "gnus-uu"))
(put 'dired-keys 'custom-loads '("dired-x"))
(put 'lisp-mnt 'custom-loads '("lisp-mnt"))
(put 'ediff-mult 'custom-loads '("ediff-mult"))
(put 'mpuz 'custom-loads '("mpuz"))
(put 'find-file 'custom-loads '("files" "find-file"))
(put 'fortran-comment 'custom-loads '("fortran"))
(put 'idlwave-online-help 'custom-loads '("idlwave"))
(put 'viper 'custom-loads '("viper-ex" "viper-init" "viper-keym" "viper-macs" "viper-mous" "viper"))
(put 'ps-print-page 'custom-loads '("ps-print"))
(put 'postscript 'custom-loads '("ps-print" "ebnf2ps"))
(put 'gnus-undo 'custom-loads '("gnus-undo"))
(put 'widget-faces 'custom-loads '("wid-edit"))
(put 'resize-minibuffer 'custom-loads '("rsz-mini"))
(put 'info-lookup 'custom-loads '("info-look"))
(put 'show-tabs 'custom-loads '("generic-x"))
(put 'gnus-various 'custom-loads '("gnus-sum" "gnus"))
(put 'elide-head 'custom-loads '("elide-head"))
(put 'vhdl-compile 'custom-loads '("vhdl-mode"))
(put 'ebrowse-tree 'custom-loads '("ebrowse"))
(put 'font-lock-highlighting-faces 'custom-loads '("font-lock" "vhdl-mode"))
(put 'flyspell 'custom-loads '("flyspell"))
(put 'ange-ftp 'custom-loads '("ange-ftp"))
(put 'font-lock-extra-types 'custom-loads '("font-lock"))
(put 'ebnf-shape 'custom-loads '("ebnf2ps"))
(put 'minibuffer 'custom-loads '("simple" "complete" "icomplete"))
(put 'rmail-edit 'custom-loads '("rmailedit"))
(put 'gnus-meta 'custom-loads '("gnus"))
(put 'mouse-sel 'custom-loads '("mouse-sel"))
(put 'sort 'custom-loads '("sort"))
(put 'antlr 'custom-loads '("antlr-mode"))
(put 'mail-source 'custom-loads '("mail-source"))
(put 'trace 'custom-loads '("trace"))
(put 'gnus-article-washing 'custom-loads '("gnus-art"))
(put 'mh-hook 'custom-loads '("mh-e"))
(put 'yow 'custom-loads '("yow"))
(put 'reftex-defining-label-environments 'custom-loads '("reftex-vars"))
(put 'asm 'custom-loads '("asm-mode"))
(put 'gnus-score-files 'custom-loads '("gnus-score" "gnus"))
(put 'mail-abbrev 'custom-loads '("mailabbrev"))
(put 'feedmail 'custom-loads '("feedmail"))
(put 'gnus-agent 'custom-loads '("gnus-agent"))
(put 'message-news 'custom-loads '("message"))
(put 'eshell 'custom-loads '("esh-arg" "esh-cmd" "esh-ext" "esh-io" "esh-mode" "esh-module" "esh-opt" "esh-proc" "esh-test" "esh-util" "esh-var" "eshell"))
(put 'bib 'custom-loads '("bib-mode"))
(put 'vhdl-align 'custom-loads '("vhdl-mode"))
(put 'iswitchb 'custom-loads '("iswitchb"))
(put 'custom-buffer 'custom-loads '("cus-edit"))
(put 'vhdl-header 'custom-loads '("vhdl-mode"))
(put 'eshell-cmd 'custom-loads '("esh-cmd"))
(put 'tex-run 'custom-loads '("tex-mode"))
(put 'reftex-finding-files 'custom-loads '("reftex-vars"))
(put 'iso-acc 'custom-loads '("iso-acc"))
(put 'gnus-summary-pick 'custom-loads '("gnus-salt"))
(put 'gnus-thread 'custom-loads '("gnus-sum"))
(put 'languages 'custom-loads '("cus-edit" "info-look" "ada-mode" "antlr-mode" "asm-mode" "cperl-mode" "dcl-mode" "delphi" "f90" "fortran" "hideshow" "icon" "idlwave" "m4-mode" "meta-mode" "modula2" "octave-mod" "pascal" "perl-mode" "prolog" "ps-mode" "sh-script" "simula" "tcl" "vhdl-mode" "sgml-mode"))
(put 'reftex-miscellaneous-configurations 'custom-loads '("reftex-vars"))
(put 'pong 'custom-loads '("pong"))
(put 'ediff-ptch 'custom-loads '("ediff-ptch"))
(put 'jka-compr 'custom-loads '("jka-compr"))
(put 'development 'custom-loads '("cus-edit"))
(put 'ediff 'custom-loads '("ediff-diff" "ediff-init" "ediff-mult" "ediff-ptch" "ediff-wind" "ediff"))
(put 'feedmail-spray 'custom-loads '("feedmail"))
(put 'idlwave-external-programs 'custom-loads '("idlwave"))
(put 'strokes 'custom-loads '("strokes"))
(put 'mouse 'custom-loads '("faces" "mouse" "menu-bar" "avoid" "mouse-sel" "msb" "mwheel" "strokes" "tooltip" "goto-addr" "artist"))
(put 'nnmail-various 'custom-loads '("nnmail"))
(put 'smiley 'custom-loads '("smiley-ems"))
(put 'extensions 'custom-loads '("generic" "time-stamp" "wid-edit" "cust-print" "eldoc" "page-ext"))
(put 'ebnf-displacement 'custom-loads '("ebnf2ps"))
(put 'appt 'custom-loads '("appt"))
(put 'snmp 'custom-loads '("snmp-mode"))
(put 'speedbar-faces 'custom-loads '("speedbar" "vhdl-mode"))
(put 'rmail 'custom-loads '("paths" "rmail" "undigest"))
(put 'ps-print-n-up 'custom-loads '("ps-print"))
(put 'eshell-arg 'custom-loads '("esh-arg"))
(put 'ps-print-printer 'custom-loads '("ps-print"))
(put 'message-various 'custom-loads '("message"))
(put 'term 'custom-loads '("terminal" "term"))
(put 'gnus-summary-exit 'custom-loads '("gnus-sum" "gnus"))
(put 'news 'custom-loads '("gnus" "mm-decode" "message" "supercite"))
(put 'ada 'custom-loads '("ada-mode" "ada-xref"))
(put 'gnus 'custom-loads '("gnus-art" "gnus-async" "gnus-demon" "gnus-dup" "gnus-eform" "gnus-undo" "gnus-uu" "gnus-win" "gnus" "mail-source" "nnmail" "rfc1843"))
(put 'cperl-affected-by-hairy 'custom-loads '("cperl-mode"))
(put 'eudc-ldap 'custom-loads '("eudc-vars"))
(put 'reftex-making-and-inserting-labels 'custom-loads '("reftex-vars"))
(put 'calendar-hooks 'custom-loads '("calendar"))
(put 'mailalias 'custom-loads '("mailalias"))
(put 'frames 'custom-loads '("frame" "faces" "scroll-bar" "menu-bar" "desktop" "ediff-wind" "imenu" "rsz-mini" "two-column"))
(put 'gnus-server 'custom-loads '("gnus"))
(put 'paren-showing 'custom-loads '("paren"))
(put 'dos-fns 'custom-loads '("dos-vars"))
(put 'display-time 'custom-loads '("time"))
(put 'multimedia 'custom-loads '("image" "gnus-audio" "mm-decode"))
(put 'reftex 'custom-loads '("reftex-vars"))
(put 'makemode 'custom-loads '("make-mode"))
(put 'hexl 'custom-loads '("hexl"))
(put 'gnus-summary-visual 'custom-loads '("gnus-sum" "gnus"))
(put 'eudc-ph 'custom-loads '("eudc-vars"))
(put 'gnus-group-listing 'custom-loads '("gnus-group"))
(put 'rmail-reply 'custom-loads '("rmail"))
(put 'gnus-score 'custom-loads '("gnus-nocem" "gnus"))
(put 'gnus-group-select 'custom-loads '("gnus-sum" "gnus"))
(put 'archive-lzh 'custom-loads '("arc-mode"))
(put 'eshell-prompt 'custom-loads '("em-prompt"))
(put 'vhdl-sequential-process 'custom-loads '("vhdl-mode"))
(put 'simula 'custom-loads '("simula"))
(put 'vhdl-comment 'custom-loads '("vhdl-mode"))
(put 'message-buffers 'custom-loads '("message"))
(put '5x5 'custom-loads '("5x5"))
(put 'generic 'custom-loads '("generic-x" "generic"))
(put 'docs 'custom-loads '("info" "makeinfo" "texinfo"))
(put 'indent 'custom-loads '("indent"))
(put 'eshell-alias 'custom-loads '("em-alias"))
(put 'enriched 'custom-loads '("enriched"))
(put 'gnus-threading 'custom-loads '("gnus-sum"))
(put 'hide-ifdef 'custom-loads '("hideif"))
(put 'vip 'custom-loads '("vip"))
(put 'smerge 'custom-loads '("smerge-mode"))
(put 'spell 'custom-loads '("spell"))
(put 'electric-help 'custom-loads '("ehelp"))
(put 'align 'custom-loads '("align"))
(put 'rmail-headers 'custom-loads '("rmail"))
(put 'timeclock 'custom-loads '("timeclock"))
(put 'gnus-score-decay 'custom-loads '("gnus-score"))
(put 'idlwave-shell-initial-commands 'custom-loads '("idlw-shell"))
(put 'tildify 'custom-loads '("tildify"))
(put 'cperl-autoinsert-details 'custom-loads '("cperl-mode"))
(put 'help 'custom-loads '("help-macro" "help" "apropos" "cus-edit" "ehelp" "info-look" "info" "man" "tooltip" "woman"))
(put 'forms 'custom-loads '("forms"))
(put 'widget-documentation 'custom-loads '("wid-edit"))
(put 'eshell-banner 'custom-loads '("em-banner"))
(put 'artist 'custom-loads '("artist"))
(put 'gnus-score-various 'custom-loads '("gnus-score" "gnus"))
(put 'cperl-faces 'custom-loads '("cperl-mode"))
(put 'goto-address 'custom-loads '("goto-addr"))
(put 'gnus-nocem 'custom-loads '("gnus-nocem"))
(put 'gnus-group-visual 'custom-loads '("gnus-group" "gnus"))
(put 'paren-matching 'custom-loads '("simple" "paren"))
(put 'font-lock 'custom-loads '("frame" "faces" "font-lock"))
(put 'auto-save 'custom-loads '("files" "startup"))
(put 'tpu 'custom-loads '("tpu-edt" "tpu-extras"))
(put 'viper-hooks 'custom-loads '("viper-init"))
(put 'gnus-cite 'custom-loads '("gnus-cite"))
(put 'gnus-demon 'custom-loads '("gnus-demon"))
(put 'reftex-optimizations-for-large-documents 'custom-loads '("reftex-vars"))
(put 'viper-misc 'custom-loads '("viper-cmd" "viper-init" "viper"))
(put 'tmm 'custom-loads '("tmm"))
(put 'gnus-message 'custom-loads '("message" "gnus"))
(put 'mh-buffer 'custom-loads '("mh-utils"))
(put 'feedmail-queue 'custom-loads '("feedmail"))
(put 'idlwave-misc 'custom-loads '("idlwave"))
(put 'cmuscheme 'custom-loads '("cmuscheme"))
(put 'ansi-colors 'custom-loads '("ansi-color"))
(put 'file-cache 'custom-loads '("filecache"))
(put 'uce 'custom-loads '("uce"))
(put 'starttls 'custom-loads '("starttls"))
(put 'boolean 'custom-loads '("bookmark"))
(put 'backup 'custom-loads '("files"))
(put 'matching 'custom-loads '("simple" "isearch" "replace" "bookmark" "completion" "ffap" "imenu"))
(put 'custom-faces 'custom-loads '("cus-edit"))
(put 'smtpmail 'custom-loads '("smtpmail"))
(put 'ebnf-syntatic 'custom-loads '("ebnf2ps"))
(put 'lisp-indent 'custom-loads '("cl-indent"))
(put 'midnight 'custom-loads '("midnight"))
(put 'tex 'custom-loads '("bibtex" "reftex-vars" "tex-mode"))
(put 'cperl-indentation-details 'custom-loads '("cperl-mode"))
(put 'mail-extr 'custom-loads '("mail-extr"))
(put 'double 'custom-loads '("double"))
(put 'imenu 'custom-loads '("imenu"))
(put 'eshell-var 'custom-loads '("esh-var"))
(put 'scribe 'custom-loads '("scribe"))
(put 'eshell-smart 'custom-loads '("em-smart"))
(put 'server 'custom-loads '("server"))
(put 'idlwave-shell-highlighting-and-faces 'custom-loads '("idlw-shell"))
(put 'tcl 'custom-loads '("tcl"))
(put 'vhdl-print 'custom-loads '("vhdl-mode"))
(put 'artist-text 'custom-loads '("artist"))
(put 'gnus-summary-tree 'custom-loads '("gnus-salt"))
(put 'tar 'custom-loads '("tar-mode"))
(put 'archive 'custom-loads '("arc-mode"))
(put 'PostScript-interaction 'custom-loads '("ps-mode"))
(put 'idlwave-abbrev-and-indent-action 'custom-loads '("idlwave"))
(put 'ps-print 'custom-loads '("ps-print"))
(put 'view 'custom-loads '("view" "calendar"))
(put 'cwarn 'custom-loads '("cwarn"))
(put 'gnus-score-default 'custom-loads '("gnus-score" "gnus-sum"))
(put 'ebnf-except 'custom-loads '("ebnf2ps"))
(put 'nnmail-duplicate 'custom-loads '("nnmail"))
(put 'handwrite 'custom-loads '("handwrite"))
(put 'tags 'custom-loads '("speedbar"))
(put 'eshell-proc 'custom-loads '("esh-proc"))
(put 'custom-browse 'custom-loads '("cus-edit"))
(put 'mime 'custom-loads '("mailcap" "mm-bodies"))
(put 'generic-x 'custom-loads '("generic-x"))
(put 'partial-completion 'custom-loads '("complete"))
(put 'whitespace 'custom-loads '("whitespace"))
(put 'maint 'custom-loads '("emacsbug" "gulp" "lisp-mnt"))
(put 'pages 'custom-loads '("page-ext"))
(put 'message-interface 'custom-loads '("message"))
(put 'diary 'custom-loads '("calendar" "diary-lib"))
(put 'custom-magic-faces 'custom-loads '("cus-edit"))
(put 'emacsbug 'custom-loads '("emacsbug"))
(put 'mh-compose 'custom-loads '("mh-comp"))
(put 'tex-view 'custom-loads '("tex-mode"))
(put 'nnmail-files 'custom-loads '("nnmail"))
(put 'gnus-edit-form 'custom-loads '("gnus-eform"))
(put 'gnus-audio 'custom-loads '("gnus-audio"))
(put 'modula2 'custom-loads '("modula2"))
(put 'ps-print-color 'custom-loads '("ps-print"))
(put 'emacs 'custom-loads '("files" "cus-edit" "ebnf2ps"))
(put 'comint 'custom-loads '("comint"))
(put 'isearch 'custom-loads '("isearch"))
(put 'reftex-fontification-configurations 'custom-loads '("reftex-vars"))
(put 'auto-insert 'custom-loads '("autoinsert"))
(put 'abbrev 'custom-loads '("files" "cus-edit" "dabbrev" "expand" "hippie-exp" "quickurl"))
(put 'ediff-merge 'custom-loads '("ediff-init" "ediff-merg"))
(put 'vhdl 'custom-loads '("vhdl-mode"))
(put 'gnus-summary-mail 'custom-loads '("gnus-sum"))
(put 'ebnf-optimization 'custom-loads '("ebnf2ps"))
(put 'apropos 'custom-loads '("apropos"))
(put 'gomoku 'custom-loads '("gomoku"))
(put 'eshell-pred 'custom-loads '("em-pred"))
(put 'tools 'custom-loads '("add-log" "calculator" "compare-w" "diff-mode" "diff" "ediff" "elide-head" "emerge" "gud" "pcvs-defs" "smerge-mode" "speedbar" "tempo" "tooltip" "vc" "which-func" "rcompile" "copyright" "compile" "ebrowse" "etags" "glasses" "make-mode"))
(put 'gnus-topic 'custom-loads '("gnus-topic"))
(put 'sgml 'custom-loads '("sgml-mode"))
(put 'keyboard 'custom-loads '("mule" "chistory" "type-break"))
(put 'isearch-faces 'custom-loads '("isearch"))
(put 'eshell-hist 'custom-loads '("em-hist"))
(put 'viper-mouse 'custom-loads '("viper-mous"))
(put 'ps-print-horizontal 'custom-loads '("ps-print"))
(put 'woman 'custom-loads '("woman"))
(put 'decipher 'custom-loads '("decipher"))
(put 'pcmpl-gnu 'custom-loads '("pcmpl-gnu"))
(put 'ps-print-face 'custom-loads '("ps-print"))
(put 'rmail-summary 'custom-loads '("rmail" "rmailsum"))
(put 'metamail 'custom-loads '("metamail"))
(put 'winner 'custom-loads '("winner"))
(put 'ebrowse-faces 'custom-loads '("ebrowse"))
(put 'wp 'custom-loads '("view" "cus-edit" "enriched" "lpr" "ps-print" "ebnf2ps" "bib-mode" "nroff-mode" "refbib" "refer" "scribe" "tildify"))
(put 'reftex-citation-support 'custom-loads '("reftex-vars"))
(put 'gnus-summary-choose 'custom-loads '("gnus-sum"))
(put 'widget-browse 'custom-loads '("wid-browse"))
(put 'feedmail-misc 'custom-loads '("feedmail"))
(put 'diff 'custom-loads '("diff-mode" "diff"))
(put 'unix 'custom-loads '("gud" "shell" "term" "rlogin" "sh-script"))
(put 'external 'custom-loads '("bib-mode" "cus-edit" "locate" "server"))
(put 'vc 'custom-loads '("files" "vc-hooks" "log-edit" "vc-cvs" "vc-rcs" "vc-sccs" "vc"))
(put 'vhdl-highlight-faces 'custom-loads '("vhdl-mode"))
(put 'which-func 'custom-loads '("which-func"))
(put 'pc-select 'custom-loads '("pc-select"))
(put 'i18n 'custom-loads '("ccl" "cus-edit" "double" "iso-acc" "iso-ascii" "latin1-disp" "ogonek"))
(put 'sh 'custom-loads '("sh-script"))
(put 'message-headers 'custom-loads '("message"))
(put 'idlwave-code-formatting 'custom-loads '("idlwave"))
(put 'basic-faces 'custom-loads '("faces" "frame"))
(put 'net-utils 'custom-loads '("net-utils"))
(put 'columns 'custom-loads '("delim-col"))
(put 'facemenu 'custom-loads '("facemenu"))
(put 'pp 'custom-loads '("pp"))
(put 'two-column 'custom-loads '("two-column"))
(put 'log-edit 'custom-loads '("log-edit"))
(put 'message-forwarding 'custom-loads '("message"))
(put 'message-faces 'custom-loads '("message"))
(put 'environment 'custom-loads '("cus-edit" "dired" "faces" "dos-vars"))
(put 'perl 'custom-loads '("perl-mode"))
(put 'vhdl-port 'custom-loads '("vhdl-mode"))
(put 'gnus-charset 'custom-loads '("gnus-group" "gnus-msg" "gnus-sum" "gnus"))
(put 'calculator 'custom-loads '("calculator"))
(put 'custom-menu 'custom-loads '("cus-edit"))
(put 'comm 'custom-loads '("eudc-vars" "ldap" "net-utils"))
(put 'type-break 'custom-loads '("type-break"))
(put 'mh 'custom-loads '("mh-comp" "mh-e" "mh-utils"))
(put 'convenience 'custom-loads '("abbrev" "autoinsert" "autorevert" "bs" "calculator" "complete" "completion" "dabbrev" "ffap" "filecache" "follow" "hippie-exp" "imenu" "iswitchb" "repeat" "speedbar" "whitespace" "windmove" "pc-select"))
(put 'lm 'custom-loads '("landmark"))
(put 'idlwave-routine-info 'custom-loads '("idlwave"))
(put 'm4 'custom-loads '("m4-mode"))
(put 'gnus-article-mime 'custom-loads '("gnus-art" "mm-uu"))
(put 'emulations 'custom-loads '("crisp" "tpu-edt" "vip" "viper"))
(put 'compression 'custom-loads '("jka-compr"))
(put 'games 'custom-loads '("5x5" "bruce" "decipher" "dunnet" "fortune" "gametree" "gomoku" "handwrite" "hanoi" "landmark" "mpuz" "pong" "solitaire" "spook" "yow"))
(put 'nnmail-retrieve 'custom-loads '("nnmail"))
(put 'gnus-duplicate 'custom-loads '("gnus-dup"))
(put 'find-function 'custom-loads '("find-func"))
(put 'menu 'custom-loads '("faces" "tmm" "easymenu"))
(put 'eshell-test 'custom-loads '("esh-test"))
(put 'vhdl-highlight 'custom-loads '("vhdl-mode"))
(put 'widgets 'custom-loads '("wid-browse" "wid-edit"))
(put 'log-view 'custom-loads '("log-view"))
(put 'PostScript 'custom-loads '("ps-mode"))
(put 'abbrev-mode 'custom-loads '("abbrev" "cus-edit" "mailabbrev"))
(put 'eshell-term 'custom-loads '("em-term"))
(put 'earcon 'custom-loads '("earcon"))
(put 'feedmail-headers 'custom-loads '("feedmail"))
(put 'hypermedia 'custom-loads '("wid-edit" "metamail" "browse-url" "goto-addr"))
(put 'image 'custom-loads '("image-file"))
(put 'ff 'custom-loads '("find-file"))
(put 'prolog 'custom-loads '("prolog"))
(put 'follow 'custom-loads '("follow"))
(put 'info 'custom-loads '("info"))
(put 'battery 'custom-loads '("battery"))
(put 'texinfo 'custom-loads '("texinfo"))
(put 'dired-mark 'custom-loads '("dired"))
(put 'lazy-lock 'custom-loads '("lazy-lock"))
(put 'makeinfo 'custom-loads '("makeinfo"))
(put 'supercite-cite 'custom-loads '("supercite"))
(put 'speedbar-vc 'custom-loads '("speedbar"))
(put 'jit-lock 'custom-loads '("jit-lock"))
(put 'save-place 'custom-loads '("saveplace"))
(put 'bs 'custom-loads '("bs"))
(put 'eldoc 'custom-loads '("eldoc"))
(put 'change-log 'custom-loads '("add-log"))
(put 'gnus-group-levels 'custom-loads '("gnus-group" "gnus-start" "gnus"))
(put 'cperl 'custom-loads '("cperl-mode"))
(put 'bs-appearance 'custom-loads '("bs"))
(put 'pcmpl-cvs 'custom-loads '("pcmpl-cvs"))
(put 'eshell-mode 'custom-loads '("esh-mode"))
(put 'files 'custom-loads '("mule" "files" "autoinsert" "autorevert" "cus-edit" "filecache" "recentf" "shadow" "ange-ftp"))
(put 'pcl-cvs 'custom-loads '("cvs-status" "log-edit" "log-view" "pcvs-defs" "pcvs-info" "pcvs-parse" "pcvs"))
(put 'rmail-files 'custom-loads '("rmail"))
(put 'gnus-summary-format 'custom-loads '("gnus-sum" "gnus"))
(put 'windmove 'custom-loads '("windmove"))
(put 'terminals 'custom-loads '("terminal"))
(put 'avoid 'custom-loads '("avoid"))
(put 'compare-w 'custom-loads '("compare-w"))
(put 'sh-indentation 'custom-loads '("sh-script"))
(put 'ebnf-production 'custom-loads '("ebnf2ps"))
(put 'gnus-files 'custom-loads '("gnus" "nnmail"))
(put 'gnus-windows 'custom-loads '("gnus-win"))
(put 'reftex-label-support 'custom-loads '("reftex-vars"))
(put 'gnus-article-buttons 'custom-loads '("gnus-art"))
(put 'gnus-summary 'custom-loads '("gnus-sum" "gnus"))
(put 'compilation 'custom-loads '("compile"))
(put 'ediff-highlighting 'custom-loads '("ediff-init"))
(put 'idlwave-shell-general-setup 'custom-loads '("idlw-shell" "idlwave"))
(put 'xscheme 'custom-loads '("xscheme"))
(put 'checkdoc 'custom-loads '("checkdoc"))
(put 'gnus-article-hiding 'custom-loads '("gnus-art" "gnus-sum"))
(put 'vhdl-mode 'custom-loads '("vhdl-mode"))
(put 'lpr 'custom-loads '("lpr"))
(put 'ispell 'custom-loads '("ispell"))
(put 'auto-revert 'custom-loads '("autorevert"))
(put 'advice 'custom-loads '("advice"))
(put 'picture 'custom-loads '("picture"))
(put 'eshell-util 'custom-loads '("esh-util"))
(put 'gnus-group 'custom-loads '("gnus-topic" "gnus"))
(put 'eudc-bbdb 'custom-loads '("eudc-vars"))
(put 'reftex-referencing-labels 'custom-loads '("reftex-vars"))
(put 'gnus-article-various 'custom-loads '("gnus-art" "gnus-sum"))
(put 'imap 'custom-loads '("imap"))
(put 'ediff-diff 'custom-loads '("ediff-diff"))
(put 'ebnf-repeat 'custom-loads '("ebnf2ps"))
(put 'supercite 'custom-loads '("supercite"))
(put 'font-selection 'custom-loads '("faces"))
(put 'ps-print-headers 'custom-loads '("ps-print"))
(put 'gnus-summary-marks 'custom-loads '("gnus-sum" "gnus"))
(put 'bibtex-autokey 'custom-loads '("bibtex"))
(put 'eudc 'custom-loads '("eudc-vars"))
(put 'octave 'custom-loads '("octave-inf" "octave-mod"))
(put 'editing-basics 'custom-loads '("simple" "files" "pc-select"))
(put 'hscroll 'custom-loads '("hscroll"))
(put 'fortune-signature 'custom-loads '("fortune"))
(put 'nnmail-expire 'custom-loads '("gnus" "nnmail"))
(put 'gnus-article-saving 'custom-loads '("gnus-art"))
(put 'icomplete 'custom-loads '("icomplete"))
(put 'LaTeX 'custom-loads '("reftex-vars"))
(put 'scrolling 'custom-loads '("frame"))
(put 'man 'custom-loads '("man"))
(put 'solitaire 'custom-loads '("solitaire"))
(put 'hippie-expand 'custom-loads '("hippie-exp"))
(put 'refer 'custom-loads '("refer"))
(put 'message-mail 'custom-loads '("message"))
(put 'modeline 'custom-loads '("faces" "time"))
(put 'archive-zoo 'custom-loads '("arc-mode"))
(put 'idlwave-completion 'custom-loads '("idlwave"))
(put 'eshell-rebind 'custom-loads '("em-rebind"))
(put 'bibtex 'custom-loads '("bibtex"))
(put 'faces 'custom-loads '("loaddefs" "faces" "facemenu" "cus-edit" "font-lock" "hilit-chg" "paren" "ps-print" "speedbar" "time" "wid-edit" "woman" "gnus" "message" "cwarn" "make-mode"))
(put 'gnus-summary-various 'custom-loads '("gnus-sum"))
(put 'applications 'custom-loads '("calendar" "cus-edit" "uniquify" "eshell" "spell"))
(put 'ebrowse-member 'custom-loads '("ebrowse"))
(put 'terminal 'custom-loads '("terminal"))
(put 'shadow 'custom-loads '("shadowfile" "shadow"))
(put 'hl-line 'custom-loads '("hl-line"))
(put 'eshell-glob 'custom-loads '("em-glob"))
(put 'internal 'custom-loads '("startup" "cus-edit" "delim-col"))
(put 'lisp 'custom-loads '("simple" "lisp" "lisp-mode" "ielm" "xscheme" "advice" "bytecomp" "checkdoc" "cl-indent" "cust-print" "edebug" "eldoc" "elp" "find-func" "pp" "re-builder" "shadow" "trace" "scheme"))
(put 'local 'custom-loads '("calendar"))
(put 'rlogin 'custom-loads '("rlogin"))
(put 'debugger 'custom-loads '("debug"))
(put 'archive-zip 'custom-loads '("arc-mode"))
(put 'gnus-start-server 'custom-loads '("gnus-start"))
(put 'debug 'custom-loads '("debug"))
(put 'vhdl-electric 'custom-loads '("vhdl-mode"))
(put 'gnus-extract-archive 'custom-loads '("gnus-uu"))
(put 'message 'custom-loads '("startup" "message" "footnote"))
(put 'message-sending 'custom-loads '("message"))
(put 'archive-arc 'custom-loads '("arc-mode"))
(put 'rmail-output 'custom-loads '("rmailout"))
(put 'editing 'custom-loads '("simple" "view" "indent" "paragraphs" "cus-edit" "faces" "outline" "hl-line" "vcursor" "whitespace" "auto-show" "hscroll" "picture"))
(put 'crisp 'custom-loads '("crisp"))
(put 'nroff 'custom-loads '("nroff-mode"))
(put 'gnus-group-icons 'custom-loads '("gnus-group"))
(put 'executable 'custom-loads '("executable"))
(put 'gnus-score-adapt 'custom-loads '("gnus-score" "gnus"))
(put 'copyright 'custom-loads '("copyright"))
(put 'bytecomp 'custom-loads '("bytecomp"))
(put 'message-insertion 'custom-loads '("message"))
(put 'pcmpl-unix 'custom-loads '("pcmpl-unix"))
(put 'gnus-extract-post 'custom-loads '("gnus-uu"))
(put 'reftex-viewing-cross-references 'custom-loads '("reftex-vars"))
(put 'hanoi 'custom-loads '("hanoi"))
(put 'reftex-index-support 'custom-loads '("reftex-vars"))
(put 'pascal 'custom-loads '("pascal"))
(put 'rmail-retrieve 'custom-loads '("rmail" "rmailsum"))
(put 'data 'custom-loads '("text-mode" "arc-mode" "forms" "hexl" "jka-compr" "saveplace" "sort" "tar-mode" "time-stamp" "snmp-mode" "timeclock"))
(put 'mail 'custom-loads '("simple" "startup" "time" "gnus" "mail-utils" "mm-decode" "message" "imap" "starttls" "emacsbug" "feedmail" "mail-extr" "mail-hist" "mailalias" "metamail" "mh-e" "mspools" "rmail" "sendmail" "smtpmail" "supercite" "uce" "eudc-vars" "fortune"))
(put 'paren-blinking 'custom-loads '("simple"))
(put 'gnus-summary-sort 'custom-loads '("gnus-sum"))
(put 'fast-lock 'custom-loads '("fast-lock"))
(put 'gnus-group-new 'custom-loads '("gnus-start"))
(put 'viper-highlighting 'custom-loads '("viper-init"))
(put 'customize 'custom-loads '("wid-edit" "cus-edit"))
(put 'dired 'custom-loads '("files" "ls-lisp" "dired-aux" "dired-x" "dired" "find-dired"))
(put 'recentf 'custom-loads '("recentf"))
(put 'fill 'custom-loads '("simple" "newcomment" "fill" "align"))
(put 'outlines 'custom-loads '("allout" "outline" "ooutline"))
(put 'latin1-display 'custom-loads '("latin1-disp"))
(put 'paragraphs 'custom-loads '("paragraphs"))
(put 'ebrowse 'custom-loads '("ebrowse"))
(put 'nnmail-split 'custom-loads '("nnmail"))
(put 'makefile 'custom-loads '("make-mode"))
(put 'supercite-attr 'custom-loads '("supercite"))
(put 'fortune 'custom-loads '("fortune"))
(put 'diff-mode 'custom-loads '("diff-mode"))
(put 'isearch-lazy-highlight 'custom-loads '("isearch"))
(put 'gnus-asynchronous 'custom-loads '("gnus-async"))
(put 'woman-formatting 'custom-loads '("woman"))
(put 'vcursor 'custom-loads '("vcursor"))
(put 'iso-ascii 'custom-loads '("iso-ascii"))
(put 'gnus-dribble-file 'custom-loads '("gnus-start"))
(put 'widget-button 'custom-loads '("wid-edit"))
(put 'uniquify 'custom-loads '("uniquify"))
(put 'old-c++ 'custom-loads '("cplus-md"))
(put 'ps-print-font 'custom-loads '("ps-mule" "ps-print"))
(put 'eshell-basic 'custom-loads '("em-basic"))
(put 'vhdl-misc 'custom-loads '("vhdl-mode"))
(put 'dired-x 'custom-loads '("dired-x"))
(put 'spook 'custom-loads '("spook"))
(put 'tex-file 'custom-loads '("tex-mode"))
(put 'eshell-dirs 'custom-loads '("em-dirs"))
(put 'time-stamp 'custom-loads '("time-stamp"))
(put 'todo 'custom-loads '("todo-mode"))
(put 'ebnf-special 'custom-loads '("ebnf2ps"))
(put 'gnus-article-highlight 'custom-loads '("gnus-art"))
(put 'tooltip 'custom-loads '("tooltip"))
(put 'gud 'custom-loads '("gud" "tooltip"))
(put 'c-macro 'custom-loads '("cmacexp"))
(put 'gnus-cache 'custom-loads '("gnus-cache" "gnus"))
(put 'eshell-module 'custom-loads '("esh-groups" "esh-module"))
(put 'gnus-extract 'custom-loads '("gnus" "gnus-uu" "uudecode"))
(put 'quickurl 'custom-loads '("quickurl"))
(put 'browse-url 'custom-loads '("browse-url"))
(put 'cust-print 'custom-loads '("cust-print"))
(put 'gnus-article 'custom-loads '("gnus-art" "gnus-cite"))
(put 'fortran-indent 'custom-loads '("fortran"))
(put 'comment 'custom-loads '("newcomment"))
(put 'hardware 'custom-loads '("battery"))
(put 'edebug 'custom-loads '("edebug"))
(put 'emerge 'custom-loads '("emerge"))
(put 'scheme 'custom-loads '("cmuscheme" "scheme"))
(put 'gametree 'custom-loads '("gametree"))
(put 'gnus-group-foreign 'custom-loads '("gnus-group"))
(put 'initialization 'custom-loads '("startup"))
(put 'sh-script 'custom-loads '("sh-script"))
(put 'cperl-speed 'custom-loads '("cperl-mode"))
(put 'PostScript-edit 'custom-loads '("ps-mode"))
(put 'calendar 'custom-loads '("midnight" "calendar" "solar" "todo-mode"))
(put 'programming 'custom-loads '("cus-edit"))
(put 'meta-font 'custom-loads '("meta-mode"))
(put 'ps-print-zebra 'custom-loads '("ps-print"))
(put 'eshell-unix 'custom-loads '("em-unix"))
(put 'hi-lock-faces 'custom-loads '("hi-lock"))
(put 'hideshow 'custom-loads '("hideshow"))
(put 'mailcap 'custom-loads '("mailcap"))
(put 'viper-search 'custom-loads '("viper-init"))
(put 'ls-lisp 'custom-loads '("ls-lisp"))
(put 'mule 'custom-loads '("mule" "mule-cmds" "kkc"))
(put 'glasses 'custom-loads '("glasses"))
(put 'vhdl-style 'custom-loads '("vhdl-mode"))
(put 'tempo 'custom-loads '("tempo"))
(put 'c 'custom-loads '("cc-vars" "cmacexp" "cpp" "hideif"))
(put 'nnmail-prepare 'custom-loads '("nnmail"))
(put 'processes 'custom-loads '("ansi-color" "comint" "cus-edit" "executable" "pcomplete" "shell" "term" "metamail" "rcompile" "rlogin" "compile" "sql" "flyspell"))
(put 'ebnf2ps 'custom-loads '("ebnf2ps"))
(put 'sendmail 'custom-loads '("sendmail"))
(put 'gnus-article-signature 'custom-loads '("gnus-art"))
(put 'eshell-ls 'custom-loads '("em-ls"))
(put 'idlwave 'custom-loads '("idlw-shell" "idlwave"))
(put 'viper-ex 'custom-loads '("viper-ex"))
(put 'gulp 'custom-loads '("gulp"))
(put 'leim 'custom-loads '("quail"))
(put 'ielm 'custom-loads '("ielm"))
(put 'find-dired 'custom-loads '("find-dired"))
(put 'delphi 'custom-loads '("delphi"))
(put 're-builder 'custom-loads '("re-builder"))
(put 'eshell-io 'custom-loads '("esh-io"))
(put 'killing 'custom-loads '("simple" "x-win"))
(put 'woman-interface 'custom-loads '("woman"))
(put 'gnus-group-various 'custom-loads '("gnus-group" "gnus"))
;;; These are for handling :version.  We need to have a minimum of
;;; information so `custom-changed-variables' could do its job.  
;;; For both groups and variables we have to set `custom-version'.
;;; For variables we also set the `standard-value' and for groups
;;; `group-documentation' (which is shown in the customize buffer), so
;;; we don't have to load the file containing the group.

;;; `custom-versions-load-alist' is an alist that has as car a version
;;; number and as elts the files that have variables that contain that
;;; version. These files should be loaded before showing the
;;; customization buffer that `customize-changed-options' generates.


;;; This macro is used so we don't modify the information about
;;; variables and groups if it's already set. (We don't know when
;;; cus-load.el is going to be loaded and at that time some of the
;;; files might be loaded and some others might not).
(defmacro custom-put-if-not (symbol propname value)
  `(unless (get ,symbol ,propname)
     (put ,symbol ,propname ,value)))

(custom-put-if-not 'mime-display 'custom-version "21.1")
(custom-put-if-not 'mime-display 'group-documentation "Display of MIME in mail and news articles.")
(custom-put-if-not 'SQL 'custom-version "20.4")
(custom-put-if-not 'SQL 'group-documentation "Running a SQL interpreter from within Emacs buffers")
(custom-put-if-not 'sql-postgres-options 'custom-version "20.8")
(custom-put-if-not 'sql-postgres-options 'standard-value t)
(custom-put-if-not 'speedbar-reconfigure-keymaps-hook 'custom-version "21.1")
(custom-put-if-not 'speedbar-reconfigure-keymaps-hook 'standard-value t)
(custom-put-if-not 'highlight-changes 'custom-version "20.4")
(custom-put-if-not 'highlight-changes 'group-documentation "Highlight Changes mode.")
(custom-put-if-not 'browse-url-gnome-moz-arguments 'custom-version "21.1")
(custom-put-if-not 'browse-url-gnome-moz-arguments 'standard-value t)
(custom-put-if-not 'query-replace-from-history-variable 'custom-version "20.3")
(custom-put-if-not 'query-replace-from-history-variable 'standard-value t)
(custom-put-if-not 'footnote 'custom-version "21.1")
(custom-put-if-not 'footnote 'group-documentation "Support for footnotes in mail and news messages.")
(custom-put-if-not 'pcomplete 'custom-version "21.1")
(custom-put-if-not 'pcomplete 'group-documentation "Programmable completion.")
(custom-put-if-not 'gnus-mime-action-alist 'custom-version "21.1")
(custom-put-if-not 'gnus-mime-action-alist 'standard-value t)
(custom-put-if-not 'fancy-splash-screen 'custom-version "21.1")
(custom-put-if-not 'fancy-splash-screen 'group-documentation "Fancy splash screen when Emacs starts.")
(custom-put-if-not 'flyspell-large-region 'custom-version "21.1")
(custom-put-if-not 'flyspell-large-region 'standard-value t)
(custom-put-if-not 'custom-variable-default-form 'custom-version "20.3")
(custom-put-if-not 'custom-variable-default-form 'standard-value t)
(custom-put-if-not 'gnus-agent-synchronize-flags 'custom-version "21.1")
(custom-put-if-not 'gnus-agent-synchronize-flags 'standard-value t)
(custom-put-if-not 'mode-line 'custom-version "21.1")
(custom-put-if-not 'mode-line 'group-documentation nil)
(custom-put-if-not 'find-function-after-hook 'custom-version "20.3")
(custom-put-if-not 'find-function-after-hook 'standard-value t)
(custom-put-if-not 'flyspell-sort-corrections 'custom-version "21.1")
(custom-put-if-not 'flyspell-sort-corrections 'standard-value t)
(custom-put-if-not 'add-log-keep-changes-together 'custom-version "20.3")
(custom-put-if-not 'add-log-keep-changes-together 'standard-value t)
(custom-put-if-not 'eval-expression-debug-on-error 'custom-version "21.1")
(custom-put-if-not 'eval-expression-debug-on-error 'standard-value t)
(custom-put-if-not 'browse-url-lynx-input-field 'custom-version "20.3")
(custom-put-if-not 'browse-url-lynx-input-field 'standard-value t)
(custom-put-if-not 'help-highlight-face 'custom-version "20.3")
(custom-put-if-not 'help-highlight-face 'standard-value t)
(custom-put-if-not 'ldap 'custom-version "21.1")
(custom-put-if-not 'ldap 'group-documentation "Lightweight Directory Access Protocol.")
(custom-put-if-not 'vc-cvs-header 'custom-version "21.1")
(custom-put-if-not 'vc-cvs-header 'standard-value t)
(custom-put-if-not 'ps-footer-frame-alist 'custom-version "21.1")
(custom-put-if-not 'ps-footer-frame-alist 'standard-value t)
(custom-put-if-not 'speedbar 'custom-version "20.3")
(custom-put-if-not 'speedbar 'group-documentation "File and tag browser frame.")
(custom-put-if-not 'message-dont-reply-to-names 'custom-version "21.1")
(custom-put-if-not 'message-dont-reply-to-names 'standard-value t)
(custom-put-if-not 'nnmail-expiry-target 'custom-version "21.1")
(custom-put-if-not 'nnmail-expiry-target 'standard-value t)
(custom-put-if-not 'custom-button-face 'custom-version "21.1")
(custom-put-if-not 'custom-button-face 'group-documentation nil)
(custom-put-if-not 'ps-header-frame-alist 'custom-version "21.1")
(custom-put-if-not 'ps-header-frame-alist 'standard-value t)
(custom-put-if-not 'flyspell-duplicate-distance 'custom-version "21.1")
(custom-put-if-not 'flyspell-duplicate-distance 'standard-value t)
(custom-put-if-not 'cursor 'custom-version "21.1")
(custom-put-if-not 'cursor 'group-documentation "Displaying text cursors.")
(custom-put-if-not 'change-log-date-face 'custom-version "21.1")
(custom-put-if-not 'change-log-date-face 'group-documentation nil)
(custom-put-if-not 'find-tag-marker-ring-length 'custom-version "20.3")
(custom-put-if-not 'find-tag-marker-ring-length 'standard-value t)
(custom-put-if-not 'elide-head 'custom-version "21.1")
(custom-put-if-not 'elide-head 'group-documentation "Eliding copyright headers and the like in source files.")
(custom-put-if-not 'ps-line-number-color 'custom-version "21.1")
(custom-put-if-not 'ps-line-number-color 'standard-value t)
(custom-put-if-not 'confirm-kill-emacs 'custom-version "21.1")
(custom-put-if-not 'confirm-kill-emacs 'standard-value t)
(custom-put-if-not 'vc-rcs-checkout-switches 'custom-version "21.1")
(custom-put-if-not 'vc-rcs-checkout-switches 'standard-value t)
(custom-put-if-not 'change-log-conditionals-face 'custom-version "21.1")
(custom-put-if-not 'change-log-conditionals-face 'group-documentation nil)
(custom-put-if-not 'mail-source 'custom-version "21.1")
(custom-put-if-not 'mail-source 'group-documentation "The mail-fetching library.")
(custom-put-if-not 'speedbar-visiting-tag-hook 'custom-version "21.1")
(custom-put-if-not 'speedbar-visiting-tag-hook 'standard-value t)
(custom-put-if-not 'eshell 'custom-version "21.1")
(custom-put-if-not 'eshell 'group-documentation "Eshell is a command shell implemented entirely in Emacs Lisp.  It
invokes no external processes beyond those requested by the user.  It
is intended to be a functional replacement for command shells such as
bash, zsh, rc, 4dos; since Emacs itself is capable of handling most of
the tasks accomplished by such tools.")
(custom-put-if-not 'compilation-scroll-output 'custom-version "20.3")
(custom-put-if-not 'compilation-scroll-output 'standard-value t)
(custom-put-if-not 'vc-cvs-diff-switches 'custom-version "21.1")
(custom-put-if-not 'vc-cvs-diff-switches 'standard-value t)
(custom-put-if-not 'gnus-unbuttonized-mime-types 'custom-version "21.1")
(custom-put-if-not 'gnus-unbuttonized-mime-types 'standard-value t)
(custom-put-if-not 'ps-right-footer 'custom-version "21.1")
(custom-put-if-not 'ps-right-footer 'standard-value t)
(custom-put-if-not 'vc-dired-recurse 'custom-version "20.3")
(custom-put-if-not 'vc-dired-recurse 'standard-value t)
(custom-put-if-not 'ps-begin-cut-regexp 'custom-version "21.1")
(custom-put-if-not 'ps-begin-cut-regexp 'standard-value t)
(custom-put-if-not 'flyspell-deplacement-commands 'custom-version "21.1")
(custom-put-if-not 'flyspell-deplacement-commands 'standard-value t)
(custom-put-if-not 'change-log-email-face 'custom-version "21.1")
(custom-put-if-not 'change-log-email-face 'group-documentation nil)
(custom-put-if-not 'mouse 'custom-version "21.1")
(custom-put-if-not 'mouse 'group-documentation "Input from the mouse.")
(custom-put-if-not 'browse-url-kde-program 'custom-version "21.1")
(custom-put-if-not 'browse-url-kde-program 'standard-value t)
(custom-put-if-not 'snmp 'custom-version "20.4")
(custom-put-if-not 'snmp 'group-documentation "Mode for editing SNMP MIB files.")
(custom-put-if-not 'widget-image-enable 'custom-version "21.1")
(custom-put-if-not 'widget-image-enable 'standard-value t)
(custom-put-if-not 'ps-line-spacing 'custom-version "21.1")
(custom-put-if-not 'ps-line-spacing 'standard-value t)
(custom-put-if-not 'help-highlight-p 'custom-version "20.3")
(custom-put-if-not 'help-highlight-p 'standard-value t)
(custom-put-if-not 'ps-paragraph-spacing 'custom-version "21.1")
(custom-put-if-not 'ps-paragraph-spacing 'standard-value t)
(custom-put-if-not 'flyspell-incorrect-hook 'custom-version "21.1")
(custom-put-if-not 'flyspell-incorrect-hook 'standard-value t)
(custom-put-if-not 'browse-url-mosaic-program 'custom-version "20.3")
(custom-put-if-not 'browse-url-mosaic-program 'standard-value t)
(custom-put-if-not 'vc-cvs-register-switches 'custom-version "21.1")
(custom-put-if-not 'vc-cvs-register-switches 'standard-value t)
(custom-put-if-not 'flyspell-tex-command-regexp 'custom-version "21.1")
(custom-put-if-not 'flyspell-tex-command-regexp 'standard-value t)
(custom-put-if-not 'flyspell-dictionaries-that-consider-dash-as-word-delimiter 'custom-version "21.1")
(custom-put-if-not 'flyspell-dictionaries-that-consider-dash-as-word-delimiter 'standard-value t)
(custom-put-if-not 'sql-oracle-options 'custom-version "20.8")
(custom-put-if-not 'sql-oracle-options 'standard-value t)
(custom-put-if-not 'find-function-regexp 'custom-version "21.1")
(custom-put-if-not 'find-function-regexp 'standard-value t)
(custom-put-if-not 'ps-footer-lines 'custom-version "21.1")
(custom-put-if-not 'ps-footer-lines 'standard-value t)
(custom-put-if-not 'vcursor-string 'custom-version "20.3")
(custom-put-if-not 'vcursor-string 'standard-value t)
(custom-put-if-not 'custom-face-default-form 'custom-version "20.3")
(custom-put-if-not 'custom-face-default-form 'standard-value t)
(custom-put-if-not 'hexl-follow-ascii 'custom-version "20.3")
(custom-put-if-not 'hexl-follow-ascii 'standard-value t)
(custom-put-if-not 'vc-cvs-use-edit 'custom-version "21.1")
(custom-put-if-not 'vc-cvs-use-edit 'standard-value t)
(custom-put-if-not 'electric-help 'custom-version "21.1")
(custom-put-if-not 'electric-help 'group-documentation "Electric help facility.")
(custom-put-if-not 'browse-url-filename-alist 'custom-version "20.3")
(custom-put-if-not 'browse-url-filename-alist 'standard-value t)
(custom-put-if-not 'align 'custom-version "21.1")
(custom-put-if-not 'align 'group-documentation "Align text to a specific column, by regexp.")
(custom-put-if-not 'ps-print-footer-frame 'custom-version "21.1")
(custom-put-if-not 'ps-print-footer-frame 'standard-value t)
(custom-put-if-not 'tildify 'custom-version "21.1")
(custom-put-if-not 'tildify 'group-documentation "Adding missing hard spaces or other text fragments into texts.")
(custom-put-if-not 'vc-rcs-checkin-switches 'custom-version "21.1")
(custom-put-if-not 'vc-rcs-checkin-switches 'standard-value t)
(custom-put-if-not 'normal-erase-is-backspace 'custom-version "21.1")
(custom-put-if-not 'normal-erase-is-backspace 'standard-value t)
(custom-put-if-not 'gnus-article-banner-alist 'custom-version "21.1")
(custom-put-if-not 'gnus-article-banner-alist 'standard-value t)
(custom-put-if-not 'region 'custom-version "21.1")
(custom-put-if-not 'region 'group-documentation nil)
(custom-put-if-not 'change-log-version-info-enabled 'custom-version "21.1")
(custom-put-if-not 'change-log-version-info-enabled 'standard-value t)
(custom-put-if-not 'find-file-suppress-same-file-warnings 'custom-version "21.1")
(custom-put-if-not 'find-file-suppress-same-file-warnings 'standard-value t)
(custom-put-if-not 'change-log-name-face 'custom-version "21.1")
(custom-put-if-not 'change-log-name-face 'group-documentation nil)
(custom-put-if-not 'sql-electric-stuff 'custom-version "20.8")
(custom-put-if-not 'sql-electric-stuff 'standard-value t)
(custom-put-if-not 'smtpmail-warn-about-unknown-extensions 'custom-version "21.1")
(custom-put-if-not 'smtpmail-warn-about-unknown-extensions 'standard-value t)
(custom-put-if-not 'gnus-ignored-from-addresses 'custom-version "21.1")
(custom-put-if-not 'gnus-ignored-from-addresses 'standard-value t)
(custom-put-if-not 'ansi-colors 'custom-version "21.1")
(custom-put-if-not 'ansi-colors 'group-documentation "Translating SGR control sequences to faces.
This translation effectively colorizes strings and regions based upon
SGR control sequences embedded in the text.  SGR (Select Graphic
Rendition) control sequences are defined in section 3.8.117 of the
ECMA-48 standard (identical to ISO/IEC 6429), which is freely available
as a PDF file <URL:http://www.ecma.ch/ecma1/STAND/ECMA-048.HTM>.")
(custom-put-if-not 'gnus-group-name-charset-method-alist 'custom-version "21.1")
(custom-put-if-not 'gnus-group-name-charset-method-alist 'standard-value t)
(custom-put-if-not 'starttls 'custom-version "21.1")
(custom-put-if-not 'starttls 'group-documentation "Support for `Transport Layer Security' protocol.")
(custom-put-if-not 'vc-sccs-register-switches 'custom-version "21.1")
(custom-put-if-not 'vc-sccs-register-switches 'standard-value t)
(custom-put-if-not 'gnus-article-date-lapsed-new-header 'custom-version "21.1")
(custom-put-if-not 'gnus-article-date-lapsed-new-header 'standard-value t)
(custom-put-if-not 'flyspell-abbrev-p 'custom-version "21.1")
(custom-put-if-not 'flyspell-abbrev-p 'standard-value t)
(custom-put-if-not 'midnight 'custom-version "20.3")
(custom-put-if-not 'midnight 'group-documentation "Run something every day at midnight.")
(custom-put-if-not 'flyspell-default-dictionary 'custom-version "21.1")
(custom-put-if-not 'flyspell-default-dictionary 'standard-value t)
(custom-put-if-not 'gnus-extra-headers 'custom-version "21.1")
(custom-put-if-not 'gnus-extra-headers 'standard-value t)
(custom-put-if-not 'speedbar-use-images 'custom-version "21.1")
(custom-put-if-not 'speedbar-use-images 'standard-value t)
(custom-put-if-not 'flyspell-default-delayed-commands 'custom-version "21.1")
(custom-put-if-not 'flyspell-default-delayed-commands 'standard-value t)
(custom-put-if-not 'gnus-auto-expirable-marks 'custom-version "21.1")
(custom-put-if-not 'gnus-auto-expirable-marks 'standard-value t)
(custom-put-if-not 'sql-interbase-options 'custom-version "20.8")
(custom-put-if-not 'sql-interbase-options 'standard-value t)
(custom-put-if-not 'automatic-hscrolling 'custom-version "21.1")
(custom-put-if-not 'automatic-hscrolling 'standard-value t)
(custom-put-if-not 'custom-buffer-done-function 'custom-version "21.1")
(custom-put-if-not 'custom-buffer-done-function 'standard-value t)
(custom-put-if-not 'gnus-article-mime-match-handle-function 'custom-version "21.1")
(custom-put-if-not 'gnus-article-mime-match-handle-function 'standard-value t)
(custom-put-if-not 'sql-db2-options 'custom-version "20.8")
(custom-put-if-not 'sql-db2-options 'standard-value t)
(custom-put-if-not 'cwarn 'custom-version "21.1")
(custom-put-if-not 'cwarn 'group-documentation "Highlight suspicious C and C++ constructions.")
(custom-put-if-not 'message-buffer-naming-style 'custom-version "21.1")
(custom-put-if-not 'message-buffer-naming-style 'standard-value t)
(custom-put-if-not 'ps-footer-font-size 'custom-version "21.1")
(custom-put-if-not 'ps-footer-font-size 'standard-value t)
(custom-put-if-not 'tags-apropos-additional-actions 'custom-version "21.1")
(custom-put-if-not 'tags-apropos-additional-actions 'standard-value t)
(custom-put-if-not 'generic-x 'custom-version "20.3")
(custom-put-if-not 'generic-x 'group-documentation "Extra modes for generic mode.")
(custom-put-if-not 'whitespace 'custom-version "21.1")
(custom-put-if-not 'whitespace 'group-documentation "Check for and fix five different types of whitespaces in source code.")
(custom-put-if-not 'debugger-mode-hook 'custom-version "20.3")
(custom-put-if-not 'debugger-mode-hook 'standard-value t)
(custom-put-if-not 'scroll-bar 'custom-version "21.1")
(custom-put-if-not 'scroll-bar 'group-documentation nil)
(custom-put-if-not 'backward-delete-char-untabify-method 'custom-version "20.3")
(custom-put-if-not 'backward-delete-char-untabify-method 'standard-value t)
(custom-put-if-not 'vc-diff-switches 'custom-version "21.1")
(custom-put-if-not 'vc-diff-switches 'standard-value t)
(custom-put-if-not 'vcursor-interpret-input 'custom-version "20.3")
(custom-put-if-not 'vcursor-interpret-input 'standard-value t)
(custom-put-if-not 'gnus-audio 'custom-version "21.1")
(custom-put-if-not 'gnus-audio 'group-documentation "Playing sound in Gnus.")
(custom-put-if-not 'trailing-whitespace 'custom-version "21.1")
(custom-put-if-not 'trailing-whitespace 'group-documentation nil)
(custom-put-if-not 'fortran-comment-line-start 'custom-version "21.1")
(custom-put-if-not 'fortran-comment-line-start 'standard-value t)
(custom-put-if-not 'fringe 'custom-version "21.1")
(custom-put-if-not 'fringe 'group-documentation nil)
(custom-put-if-not 'vc-rcs-header 'custom-version "21.1")
(custom-put-if-not 'vc-rcs-header 'standard-value t)
(custom-put-if-not 'vhdl 'custom-version "20.4")
(custom-put-if-not 'vhdl 'group-documentation "Customizations for VHDL Mode.")
(custom-put-if-not 'message-mail-user-agent 'custom-version "21.1")
(custom-put-if-not 'message-mail-user-agent 'standard-value t)
(custom-put-if-not 'face-font-registry-alternatives 'custom-version "21.1")
(custom-put-if-not 'face-font-registry-alternatives 'standard-value t)
(custom-put-if-not 'show-paren-priority 'custom-version "21.1")
(custom-put-if-not 'show-paren-priority 'standard-value t)
(custom-put-if-not 'isearch-faces 'custom-version "21.1")
(custom-put-if-not 'isearch-faces 'group-documentation "Lazy highlighting feature for incremental search.")
(custom-put-if-not 'vc-handled-backends 'custom-version "21.1")
(custom-put-if-not 'vc-handled-backends 'standard-value t)
(custom-put-if-not 'find-function-recenter-line 'custom-version "20.3")
(custom-put-if-not 'find-function-recenter-line 'standard-value t)
(custom-put-if-not 'tags-tag-face 'custom-version "21.1")
(custom-put-if-not 'tags-tag-face 'standard-value t)
(custom-put-if-not 'change-log-list-face 'custom-version "21.1")
(custom-put-if-not 'change-log-list-face 'group-documentation nil)
(custom-put-if-not 'vc-rcs-register-switches 'custom-version "21.1")
(custom-put-if-not 'vc-rcs-register-switches 'standard-value t)
(custom-put-if-not 'gnus-nocem-check-article-limit 'custom-version "21.1")
(custom-put-if-not 'gnus-nocem-check-article-limit 'standard-value t)
(custom-put-if-not 'mail-citation-prefix-regexp 'custom-version "20.3")
(custom-put-if-not 'mail-citation-prefix-regexp 'standard-value t)
(custom-put-if-not 'diary-mail-days 'custom-version "20.3")
(custom-put-if-not 'diary-mail-days 'standard-value t)
(custom-put-if-not 'diary-mail-addr 'custom-version "20.3")
(custom-put-if-not 'diary-mail-addr 'standard-value t)
(custom-put-if-not 'nnmail-extra-headers 'custom-version "21.1")
(custom-put-if-not 'nnmail-extra-headers 'standard-value t)
(custom-put-if-not 'font-lock-support-mode 'custom-version "21.1")
(custom-put-if-not 'font-lock-support-mode 'standard-value t)
(custom-put-if-not 'compilation-error-screen-columns 'custom-version "20.4")
(custom-put-if-not 'compilation-error-screen-columns 'standard-value t)
(custom-put-if-not 'debugger-record-buffer 'custom-version "20.3")
(custom-put-if-not 'debugger-record-buffer 'standard-value t)
(custom-put-if-not 'query-replace-to-history-variable 'custom-version "20.3")
(custom-put-if-not 'query-replace-to-history-variable 'standard-value t)
(custom-put-if-not 'gnus-mime-multipart-functions 'custom-version "21.1")
(custom-put-if-not 'gnus-mime-multipart-functions 'standard-value t)
(custom-put-if-not 'which-func 'custom-version "20.3")
(custom-put-if-not 'which-func 'group-documentation "Mode to display the current function name in the modeline.")
(custom-put-if-not 'read-mail-command 'custom-version "21.1")
(custom-put-if-not 'read-mail-command 'standard-value t)
(custom-put-if-not 'ange-ftp-try-passive-mode 'custom-version 21.1)
(custom-put-if-not 'ange-ftp-try-passive-mode 'standard-value t)
(custom-put-if-not 'rmail-automatic-folder-directives 'custom-version "21.1")
(custom-put-if-not 'rmail-automatic-folder-directives 'standard-value t)
(custom-put-if-not 'mouse-buffer-menu-mode-mult 'custom-version "20.3")
(custom-put-if-not 'mouse-buffer-menu-mode-mult 'standard-value t)
(custom-put-if-not 'ps-left-footer 'custom-version "21.1")
(custom-put-if-not 'ps-left-footer 'standard-value t)
(custom-put-if-not 'eval-expression-print-length 'custom-version "21.1")
(custom-put-if-not 'eval-expression-print-length 'standard-value t)
(custom-put-if-not 'net-utils 'custom-version "20.3")
(custom-put-if-not 'net-utils 'group-documentation "Network utility functions.")
(custom-put-if-not 'log-edit 'custom-version "21.1")
(custom-put-if-not 'log-edit 'group-documentation "Major mode for editing RCS and CVS commit messages.")
(custom-put-if-not 'cperl-invalid-face 'custom-version "21.1")
(custom-put-if-not 'cperl-invalid-face 'standard-value t)
(custom-put-if-not 'finger-X.500-host-regexps 'custom-version "21.1")
(custom-put-if-not 'finger-X.500-host-regexps 'standard-value t)
(custom-put-if-not 'uce-mail-reader 'custom-version "20.3")
(custom-put-if-not 'uce-mail-reader 'standard-value t)
(custom-put-if-not 'gnus-charset 'custom-version "21.1")
(custom-put-if-not 'gnus-charset 'group-documentation "Group character set issues.")
(custom-put-if-not 'calculator 'custom-version "21.1")
(custom-put-if-not 'calculator 'group-documentation "Simple Emacs calculator.")
(custom-put-if-not 'custom-button-pressed-face 'custom-version "21.1")
(custom-put-if-not 'custom-button-pressed-face 'group-documentation nil)
(custom-put-if-not 'dos-codepage-setup-hook 'custom-version "20.3.3")
(custom-put-if-not 'dos-codepage-setup-hook 'standard-value t)
(custom-put-if-not 'change-log-version-number-regexp-list 'custom-version "21.1")
(custom-put-if-not 'change-log-version-number-regexp-list 'standard-value t)
(custom-put-if-not 'menu 'custom-version "21.1")
(custom-put-if-not 'menu 'group-documentation "Input from the menus.")
(custom-put-if-not 'gnus-nocem-check-from 'custom-version "21.1")
(custom-put-if-not 'gnus-nocem-check-from 'standard-value t)
(custom-put-if-not 'sgml-validate-command 'custom-version "21.1")
(custom-put-if-not 'sgml-validate-command 'standard-value t)
(custom-put-if-not 'jit-lock 'custom-version "21.1")
(custom-put-if-not 'jit-lock 'group-documentation "Font Lock support mode to fontify just-in-time.")
(custom-put-if-not 'rmail-file-name 'custom-version "21.1")
(custom-put-if-not 'rmail-file-name 'standard-value t)
(custom-put-if-not 'bs 'custom-version "21.1")
(custom-put-if-not 'bs 'group-documentation "Buffer Selection: Maintaining buffers by buffer menu.")
(custom-put-if-not 'cperl 'custom-version "20.3")
(custom-put-if-not 'cperl 'group-documentation "Major mode for editing Perl code.")
(custom-put-if-not 'focus-follows-mouse 'custom-version "20.3")
(custom-put-if-not 'focus-follows-mouse 'standard-value t)
(custom-put-if-not 'message-forward-as-mime 'custom-version "21.1")
(custom-put-if-not 'message-forward-as-mime 'standard-value t)
(custom-put-if-not 'pcl-cvs 'custom-version "21.1")
(custom-put-if-not 'pcl-cvs 'group-documentation "Special support for the CVS versioning system.")
(custom-put-if-not 'windmove 'custom-version "21.1")
(custom-put-if-not 'windmove 'group-documentation "Directional selection of windows in a frame.")
(custom-put-if-not 'strokes-char-face 'custom-version "21.1")
(custom-put-if-not 'strokes-char-face 'group-documentation nil)
(custom-put-if-not 'vc-cvs-stay-local 'custom-version "21.1")
(custom-put-if-not 'vc-cvs-stay-local 'standard-value t)
(custom-put-if-not 'fortran-comment-line-start-skip 'custom-version "21.1")
(custom-put-if-not 'fortran-comment-line-start-skip 'standard-value t)
(custom-put-if-not 'ps-print-footer 'custom-version "21.1")
(custom-put-if-not 'ps-print-footer 'standard-value t)
(custom-put-if-not 'gnus-list-identifiers 'custom-version "21.1")
(custom-put-if-not 'gnus-list-identifiers 'standard-value t)
(custom-put-if-not 'checkdoc 'custom-version "20.3")
(custom-put-if-not 'checkdoc 'group-documentation "Support for doc string checking in Emacs Lisp.")
(custom-put-if-not 'imap 'custom-version "21.1")
(custom-put-if-not 'imap 'group-documentation "Low-level IMAP issues.")
(custom-put-if-not 'change-log-acknowledgement-face 'custom-version "21.1")
(custom-put-if-not 'change-log-acknowledgement-face 'group-documentation nil)
(custom-put-if-not 'mail-abbrevs-mode 'custom-version "20.3")
(custom-put-if-not 'mail-abbrevs-mode 'standard-value t)
(custom-put-if-not 'message-forward-ignored-headers 'custom-version "21.1")
(custom-put-if-not 'message-forward-ignored-headers 'standard-value t)
(custom-put-if-not 'eudc 'custom-version "21.1")
(custom-put-if-not 'eudc 'group-documentation "Emacs Unified Directory Client.")
(custom-put-if-not 'scrolling 'custom-version "21.1")
(custom-put-if-not 'scrolling 'group-documentation "Scrolling windows.")
(custom-put-if-not 'ps-paragraph-regexp 'custom-version "21.1")
(custom-put-if-not 'ps-paragraph-regexp 'standard-value t)
(custom-put-if-not 'ps-footer-font-family 'custom-version "21.1")
(custom-put-if-not 'ps-footer-font-family 'standard-value t)
(custom-put-if-not 'sh-imenu-generic-expression 'custom-version "20.4")
(custom-put-if-not 'sh-imenu-generic-expression 'standard-value t)
(custom-put-if-not 'temp-buffer-max-height 'custom-version "20.4")
(custom-put-if-not 'temp-buffer-max-height 'standard-value t)
(custom-put-if-not 'border 'custom-version "21.1")
(custom-put-if-not 'border 'group-documentation nil)
(custom-put-if-not 'ps-black-white-faces 'custom-version "21.1")
(custom-put-if-not 'ps-black-white-faces 'standard-value t)
(custom-put-if-not 'nnmail-scan-directory-mail-source-once 'custom-version "21.1")
(custom-put-if-not 'nnmail-scan-directory-mail-source-once 'standard-value t)
(custom-put-if-not 'gnus-preserve-marks 'custom-version "21.1")
(custom-put-if-not 'gnus-preserve-marks 'standard-value t)
(custom-put-if-not 'mail-envelope-from 'custom-version "21.1")
(custom-put-if-not 'mail-envelope-from 'standard-value t)
(custom-put-if-not 'hs-minor-mode-hook 'custom-version "21.1")
(custom-put-if-not 'hs-minor-mode-hook 'standard-value t)
(custom-put-if-not 'speedbar-indentation-width 'custom-version "21.1")
(custom-put-if-not 'speedbar-indentation-width 'standard-value t)
(custom-put-if-not 'hl-line 'custom-version "21.1")
(custom-put-if-not 'hl-line 'group-documentation "Highlight the current line.")
(custom-put-if-not 'find-file-wildcards 'custom-version "20.4")
(custom-put-if-not 'find-file-wildcards 'standard-value t)
(custom-put-if-not 'custom-comment-face 'custom-version "21.1")
(custom-put-if-not 'custom-comment-face 'group-documentation nil)
(custom-put-if-not 'custom-raised-buttons 'custom-version "21.1")
(custom-put-if-not 'custom-raised-buttons 'standard-value t)
(custom-put-if-not 'gnus-article-x-face-command 'custom-version "21.1")
(custom-put-if-not 'gnus-article-x-face-command 'standard-value t)
(custom-put-if-not 'gnus-inhibit-user-auto-expire 'custom-version "21.1")
(custom-put-if-not 'gnus-inhibit-user-auto-expire 'standard-value t)
(custom-put-if-not 'show-paren-ring-bell-on-mismatch 'custom-version "20.3")
(custom-put-if-not 'show-paren-ring-bell-on-mismatch 'standard-value t)
(custom-put-if-not 'rmail-default-body-file 'custom-version "20.3")
(custom-put-if-not 'rmail-default-body-file 'standard-value t)
(custom-put-if-not 'gnus-emphasize-whitespace-regexp 'custom-version "21.1")
(custom-put-if-not 'gnus-emphasize-whitespace-regexp 'standard-value t)
(custom-put-if-not 'gnus-summary-show-article-charset-alist 'custom-version "21.1")
(custom-put-if-not 'gnus-summary-show-article-charset-alist 'standard-value t)
(custom-put-if-not 'vc-dired-listing-switches 'custom-version "21.1")
(custom-put-if-not 'vc-dired-listing-switches 'standard-value t)
(custom-put-if-not 'rmail-movemail-flags 'custom-version "20.3")
(custom-put-if-not 'rmail-movemail-flags 'standard-value t)
(custom-put-if-not 'keyboard-coding-system 'custom-version "21.1")
(custom-put-if-not 'keyboard-coding-system 'standard-value t)
(custom-put-if-not 'sql-sybase-options 'custom-version "20.8")
(custom-put-if-not 'sql-sybase-options 'standard-value t)
(custom-put-if-not 'gnus-read-newsrc-file 'custom-version "21.1")
(custom-put-if-not 'gnus-read-newsrc-file 'standard-value t)
(custom-put-if-not 'recentf 'custom-version "21.1")
(custom-put-if-not 'recentf 'group-documentation "Maintain a menu of recently opened files.")
(custom-put-if-not 'nnmail-split-header-length-limit 'custom-version "21.1")
(custom-put-if-not 'nnmail-split-header-length-limit 'standard-value t)
(custom-put-if-not 'gnus-ignored-mime-types 'custom-version "21.1")
(custom-put-if-not 'gnus-ignored-mime-types 'standard-value t)
(custom-put-if-not 'fortune 'custom-version "21.1")
(custom-put-if-not 'fortune 'group-documentation "Settings for fortune.")
(custom-put-if-not 'gnus-agent-confirmation-function 'custom-version "21.1")
(custom-put-if-not 'gnus-agent-confirmation-function 'standard-value t)
(custom-put-if-not 'dired-at-point-require-prefix 'custom-version "20.3")
(custom-put-if-not 'dired-at-point-require-prefix 'standard-value t)
(custom-put-if-not 'diff-mode 'custom-version "21.1")
(custom-put-if-not 'diff-mode 'group-documentation "Major mode for viewing/editing diffs")
(custom-put-if-not 'isearch-lazy-highlight 'custom-version "21.1")
(custom-put-if-not 'isearch-lazy-highlight 'standard-value t)
(custom-put-if-not 'sql-mysql-options 'custom-version "20.8")
(custom-put-if-not 'sql-mysql-options 'standard-value t)
(custom-put-if-not 'next-line-add-newlines 'custom-version "21.1")
(custom-put-if-not 'next-line-add-newlines 'standard-value t)
(custom-put-if-not 'dabbrev-ignored-buffer-regexps 'custom-version "21.1")
(custom-put-if-not 'dabbrev-ignored-buffer-regexps 'standard-value t)
(custom-put-if-not 'gnus-group-highlight-words-alist 'custom-version "21.1")
(custom-put-if-not 'gnus-group-highlight-words-alist 'standard-value t)
(custom-put-if-not 'dabbrev-ignored-buffer-names 'custom-version "20.3")
(custom-put-if-not 'dabbrev-ignored-buffer-names 'standard-value t)
(custom-put-if-not 'flyspell-default-deplacement-commands 'custom-version "21.1")
(custom-put-if-not 'flyspell-default-deplacement-commands 'standard-value t)
(custom-put-if-not 'ps-end-cut-regexp 'custom-version "21.1")
(custom-put-if-not 'ps-end-cut-regexp 'standard-value t)
(custom-put-if-not 'change-log-file-face 'custom-version "21.1")
(custom-put-if-not 'change-log-file-face 'group-documentation nil)
(custom-put-if-not 'diary-unknown-time 'custom-version "20.3")
(custom-put-if-not 'diary-unknown-time 'standard-value t)
(custom-put-if-not 'vc-sccs-diff-switches 'custom-version "21.1")
(custom-put-if-not 'vc-sccs-diff-switches 'standard-value t)
(custom-put-if-not 'browse-url-lynx-emacs-args 'custom-version "20.3")
(custom-put-if-not 'browse-url-lynx-emacs-args 'standard-value t)
(custom-put-if-not 'todo 'custom-version "21.1")
(custom-put-if-not 'todo 'group-documentation "Maintain a list of todo items.")
(custom-put-if-not 'tooltip 'custom-version "21.1")
(custom-put-if-not 'tooltip 'group-documentation "Customization group for the `tooltip' package.")
(custom-put-if-not 'gnus-newsgroup-ignored-charsets 'custom-version "21.1")
(custom-put-if-not 'gnus-newsgroup-ignored-charsets 'standard-value t)
(custom-put-if-not 'message-send-mail-partially-limit 'custom-version "21.1")
(custom-put-if-not 'message-send-mail-partially-limit 'standard-value t)
(custom-put-if-not 'quickurl 'custom-version "21.1")
(custom-put-if-not 'quickurl 'group-documentation "Insert an URL based on text at point in buffer.")
(custom-put-if-not 'tool-bar 'custom-version "21.1")
(custom-put-if-not 'tool-bar 'group-documentation nil)
(custom-put-if-not 'vc-default-init-version 'custom-version "20.3")
(custom-put-if-not 'vc-default-init-version 'standard-value t)
(custom-put-if-not 'flyspell-use-global-abbrev-table-p 'custom-version "21.1")
(custom-put-if-not 'flyspell-use-global-abbrev-table-p 'standard-value t)
(custom-put-if-not 'message-forward-show-mml 'custom-version "21.1")
(custom-put-if-not 'message-forward-show-mml 'standard-value t)
(custom-put-if-not 'gametree 'custom-version "20.3")
(custom-put-if-not 'gametree 'group-documentation "Manage game analysis trees in Emacs.")
(custom-put-if-not 'vc-rcs-diff-switches 'custom-version "21.1")
(custom-put-if-not 'vc-rcs-diff-switches 'standard-value t)
(custom-put-if-not 'change-log-function-face 'custom-version "21.1")
(custom-put-if-not 'change-log-function-face 'group-documentation nil)
(custom-put-if-not 'hscroll-global-mode 'custom-version "20.3")
(custom-put-if-not 'hscroll-global-mode 'standard-value t)
(custom-put-if-not 'message-default-charset 'custom-version "21.1")
(custom-put-if-not 'message-default-charset 'standard-value t)
(custom-put-if-not 'auto-save-file-name-transforms 'custom-version "21.1")
(custom-put-if-not 'auto-save-file-name-transforms 'standard-value t)
(custom-put-if-not 'tags-apropos-verbose 'custom-version "21.1")
(custom-put-if-not 'tags-apropos-verbose 'standard-value t)
(custom-put-if-not 'ps-print-upside-down 'custom-version "21.1")
(custom-put-if-not 'ps-print-upside-down 'standard-value t)
(custom-put-if-not 'mailcap 'custom-version "21.1")
(custom-put-if-not 'mailcap 'group-documentation "Definition of viewers for MIME types.")
(custom-put-if-not 'find-variable-regexp 'custom-version "21.1")
(custom-put-if-not 'find-variable-regexp 'standard-value t)
(custom-put-if-not 'rmail-edit-mode-hook 'custom-version "21.1")
(custom-put-if-not 'rmail-edit-mode-hook 'standard-value t)
(custom-put-if-not 'header-line 'custom-version "21.1")
(custom-put-if-not 'header-line 'group-documentation nil)
(custom-put-if-not 'glasses 'custom-version "21.1")
(custom-put-if-not 'glasses 'group-documentation "Make unreadable code likeThis(one) readable.")
(custom-put-if-not 'easy-menu-precalculate-equivalent-keybindings 'custom-version "20.3")
(custom-put-if-not 'easy-menu-precalculate-equivalent-keybindings 'standard-value t)
(custom-put-if-not 'debugger-batch-max-lines 'custom-version "21.1")
(custom-put-if-not 'debugger-batch-max-lines 'standard-value t)
(custom-put-if-not 'desktop-enable 'custom-version "20.3")
(custom-put-if-not 'desktop-enable 'standard-value t)
(custom-put-if-not 'vc-dired-terse-display 'custom-version "20.3")
(custom-put-if-not 'vc-dired-terse-display 'standard-value t)
(custom-put-if-not 'server-kill-new-buffers 'custom-version "21.1")
(custom-put-if-not 'server-kill-new-buffers 'standard-value t)
(custom-put-if-not 'speedbar-hide-button-brackets-flag 'custom-version "21.1")
(custom-put-if-not 'speedbar-hide-button-brackets-flag 'standard-value t)
(custom-put-if-not 'delphi 'custom-version "21.1")
(custom-put-if-not 'delphi 'group-documentation "Major mode for editing Delphi source in Emacs")
(custom-put-if-not 'eval-expression-print-level 'custom-version "21.1")
(custom-put-if-not 'eval-expression-print-level 'standard-value t)

(defvar custom-versions-load-alist '(("20.3.3" "dos-vars") (21.1 "ange-ftp") ("20.4" "files" "help" "sh-script" "compile") ("20.3" "desktop" "easymenu" "hscroll" "dabbrev" "ffap" "rmail" "paren" "mailabbrev" "frame" "uce" "mouse" "diary-lib" "sendmail" "simple" "debug" "hexl" "vcursor" "vc" "compile" "etags" "help" "browse-url" "add-log" "find-func" "cus-edit" "replace") ("21.1" "server" "debug" "rmailedit" "dabbrev" "isearch" "gnus-start" "mule" "hideshow" "sendmail" "paths" "sgml-mode" "net-utils" "cperl-mode" "rmail" "font-lock" "gnus-nocem" "vc-hooks" "paren" "faces" "fortran" "vc" "etags" "cus-edit" "frame" "vc-sccs" "gnus-group" "gnus-sum" "smtpmail" "add-log" "find-func" "wid-edit" "vc-rcs" "files" "nnmail" "message" "ps-print" "vc-cvs" "simple" "gnus-agent" "flyspell" "gnus-art" "browse-url" "speedbar") ("20.8" "sql"))
 "For internal use by custom.")

(provide 'cus-load)

;;; Local Variables:
;;; version-control: never
;;; no-byte-compile: t
;;; no-update-autoloads: t
;;; End:
;;; cus-load.el ends here
